#!/bin/bash

#This script is intented to run everything that has to be done when acquiring data.
#It writes environmental variables (temperature, ...) to the database, then runs the acquisition, generates an ntuple for the nearline analysis, and at last re-writes environmental variables to the database.

#Written by Emilien Chapon, June 5 2008
#Modified Thu Jun  5 18:04:35 CDT 2008 Ed May
#Modified Fri Jun  6 19:00 Ed May	test with series of runs

#First, set the run number. For that we read the previous run number from the database, provided it already exists.
if test -f database.db ;
then
	RUN_NUMBER=$(tail -n 1 database.db | sed -e 's/ .*//')
	RUN_NUMBER=`expr $RUN_NUMBER \+ 1`
else
	echo "warning, couldn't find the database, creating a new one"
	RUN_NUMBER=81
fi

#First read of parameters before the run
echo "Beginning of run $RUN_NUMBER, now ($(date '+%T %D')) retrieving beam environment parameters and writing them to the database"
#Check if the database is blocked
while test -f database.db.lock ; do
	echo 'warning, database is locked'
	sleep 1
done
echo > database.db.lock
echo -e $RUN_NUMBER '\t' 1 '\t' $(date '+%T') '\t' $(date '+%D') '\t' $(date '+%s') '\t' $(exec acread2 I:Beam) '\t' $(exec acread2 'F:MW1SEM') '\t' $(exec acread2 'F:MC1SEM') '\t' $(exec acread2 'F:MC6IC') '\t' $(exec acread2 'F:MT6TP1') '\t' $(exec acread2 'F:MT6TP2')  '\t' $(exec acread2 'F:MT6SC1') '\t' $(exec acread2 'F:MT6SC2')  >> database.db
rm database.db.lock
echo wrote the following line to database:
tail -n 1 database.db

# setup for beam = 0
# setup for pulser = 1
RUN_TYPE=0
#call daq program here
echo Now calling the DAQ software
cat > daq.tmp <<EOF
$RUN_TYPE
$RUN_NUMBER
EOF
# use next for background and remote operations
#daq.x < daq.tmp > daq.x.run_$RUN_NUMBER 2>daq.x.err.run_$RUN_NUMBER
# use next for online operations
daq.x < daq.tmp > daq.x.run_$RUN_NUMBER 
#Now move the stdout log file to storage and compress
mv daq.x.run_$RUN_NUMBER /data0/t979/current/
gzip /data0/t979/current/daq.x.run_$RUN_NUMBER

#Now we build ntuple for nearline analysis
#revise file pointer so ntupleBuilder.x reads correct raw data.... this needs to be fixed in ntupleBuilder.x
ln -sf data/run_$RUN_NUMBER.txt indata
./Run2Tup $RUN_NUMBER
#ntupleBuilder.x
#Change name (ntupleBuild.x should be fixed for this too)
#mv ntuple.ascii ntuple.ascii.run_$RUN_NUMBER

#Now move ntupe to storage location and compress
#mv ntuple.ascii.run_$RUN_NUMBER /data0/t979/current/
#gzip /data0/t979/current/ntuple.ascii.run_$RUN_NUMBER
cp data/run_0$RUN_NUMBER.tup /data0/t979/current/run_0$RUN_NUMBER.tup
gzip /data0/t979/current/run_0$RUN_NUMBER.tup

#Now move the raw data to storage locations and compress...
cp data/run_0$RUN_NUMBER.txt /data0/t979/current/
gzip /data0/t979/current/run_0$RUN_NUMBER.txt

#Finally, write the parameters at the end of the run
echo $(date)
echo "End of run $RUN_NUMBER, now ($(date '+%T %D')) retrieving beam environment parameters and writing them to the database"
#Check if the database is blocked
while test -f databse.db.lock ; do
	echo 'warning, database is locked'
	sleep 1
done
echo > database.db.lock
echo -e $RUN_NUMBER '\t' 3 '\t' $(date '+%T') '\t' $(date '+%D') '\t' $(date '+%s') '\t' $(exec acread2 I:Beam) '\t' $(exec acread2 'F:MW1SEM') '\t' $(exec acread2 'F:MC1SEM') '\t' $(exec acread2 'F:MC6IC') '\t' $(exec acread2 'F:MT6TP1') '\t' $(exec acread2 'F:MT6TP2')  '\t' $(exec acread2 'F:MT6SC1') '\t' $(exec acread2 'F:MT6SC2')  >> database.db
rm database.db.lock
echo wrote the following line to database:
tail -n 1 database.db

echo End of script
