// reads raw data from run_xxx.txt assigned to link indata
// writes ascii tuple file: ntuple.ascii using nout
#include <iostream>
#include <iomanip>
#include <fstream>		// needed by  ofstream to open output file
#include "histogram.h"
#include <string>

#define TAB '\t'

using namespace std;

int main(int argc, char** argv)	{

	int lineID, hdata, ddata;
	int chunkStart=3, chunkSize, chunkEnd=0xFFFF;
	int feraID;
	int lineTotalLineCount = 0;
	int fileLineCount = 0;
	int linesInChunk=0;
	int bufferNumber=0, bufferCount=0;
	string titleString;
	int ptrToNext=1;
	bool notTheFirst=false, ptrData=false, firstTime = true ;
	bool ad114Header1 = false;
	bool ad114Header2 = false;
	int ad114Header1Count=0;
	int ad114Header2Count=0;
	int header=0;
	bool tdcHeader = false;
	int tdcHeaderCount = 0;
	int seconds,useconds;
	double relativeTime = 0.0, absoluteTime = 0.0, firstAbsoluteTime = 0.0;
	double timePair[2];
	double pulserDelay = 3.0;	// 0.0, 3.0 nsec
	double pulserPeriod = 0.5;	// 2.0, 1.0, 0.5 msec
	int ntupleCount=0;
	int countNumbTS=-1;
	double timeNextPair[2];
	int runNumber=0;
	//unsigned int event[18];
	unsigned int event[28];
	int eventIndex=0, eventCount=0;
	int ad114value1=0, f4300bCh1=0;
	int ad114value2=0, f4300bCh2=0;
	int errorType1count = 0;


/*
	0	TDC header
	1	adc channel 1
	2	adc channel 2
	15	adc channel 15
	16	AD114 header 8801
	17	AD114 data

	18	AD114 header 8802
	19	AD114 data
 */

	//cout << "Enter Run Description parameters: " << endl;
	//cout <<"runNumber  pulserDelay  pulserPeriod " << endl;
	//cin >> runNumber >> pulserDelay >> pulserPeriod;


	ifstream in("indata");


	// 1st line is text not data
	//getline(in,titleString);
	//fileLineCount++;
	
	cout << "Title: " << titleString << endl;
	cout << "DEBUG:\tLine\tlineID\thdata\tddata" << endl;

   // open an output stream to make file of ascii nutples for ROOT
   ofstream nout("ntuple.ascii");
   if(!nout)
   {
     cout << "Cannot open output file.\n";
     return 1;
   }


   for (;;) {
	

	lineID = hdata = ddata = 0;
	seconds = useconds = 0;
	in >> lineID;
	if (lineID == -1) { in >> seconds >> useconds; fileLineCount++; }
	if (lineID == -2) { in >> seconds >> useconds; fileLineCount++; }
	if (lineID >=  0) { in >> hex >> hdata >> dec >> ddata; fileLineCount++; }
	if ( eventCount%100 == 0 ) {
		if (lineID < 0 ) cout << fileLineCount << TAB << lineID <<TAB
					<< seconds<<TAB<<useconds<<TAB<< endl;
		if (lineID >= 0) cout << fileLineCount << TAB << lineID <<TAB
					<<hex<<hdata<<TAB<<dec<<ddata<< endl;
	}

	// detect an EOF
	if ( in.eof() != 0 ) {
		cout << "Found end of file..." << endl;
		break;
	}

	if ( ad114Header1 ) ad114value1=ddata;

	if ( (hdata & 0x8000) == 0x8000 ) {
		// found a header
		cout << "Found a header..." << TAB << hex << hdata << endl;
		header++;
		if ( (hdata & 0x8801) == 0x8801 ) {
			// found a AD114 header
			ad114Header1 = true;
			ad114Header1Count++;
                } else if ( (hdata & 0x8802) == 0x8802 ) {
			ad114Header2 = true;
                        ad114Header2Count++;
		} else {
			cout << "Found a tdc header..." << TAB << hex << hdata << endl;
			tdcHeader = true;
			tdcHeaderCount++;
		}
		cout << "tdcHeader = " << tdcHeader << endl;
		cout << dec;
	}

	if ( tdcHeader ) eventIndex = 0;
	event[eventIndex] = ddata;
	eventIndex++;
	cout << "DEBUG eventIndex: " << eventIndex << TAB << event[eventIndex] << endl;
	if (eventIndex == 17) {
		nout << event[1] << TAB << event[2] << TAB << event[17] << endl;
		ad114Header1 = false;
		ad114Header2 = false;
		tdcHeader = false;
	} 

	
	if ( lineID < -2 ){cout << "This should never happen! Exit with error" << endl; return 1;}
	if ( lineID == -1 || lineID == -2 ) {
		cout << "Found a "<< lineID << " TimeStamp: seconds=" << seconds 
			<< " useconds=" << useconds << " firstTime = " << firstTime << endl;
		absoluteTime = seconds + useconds/1.0e+6;
		cout << "Converts to absoluteTime = " << absoluteTime << endl;
		if ( firstTime ) firstAbsoluteTime = absoluteTime;
		firstTime = false;
		relativeTime = absoluteTime - firstAbsoluteTime;
		cout << "Converts to relative time = " << relativeTime 
			<< " using firstAbsoluteTime = " << firstAbsoluteTime << endl;
		if ( lineID == -1 ) timePair[0] = relativeTime;
		if ( lineID == -2 ) timePair[1] = relativeTime;
		countNumbTS++;
		if ( (countNumbTS%4 == 0) ) timeNextPair[0] = relativeTime;
		if ( (countNumbTS%4 == 2) ) timeNextPair[1] = relativeTime;
	}


	if ( lineID >= 0 ) {
	   //in >> lineID >> hex >> hdata >> dec >> ddata;
	   lineTotalLineCount++;
	   if ( lineTotalLineCount <= 20 ) cout << "Read a line " << lineID << " " 
					<< hex << hdata << " " 
					<< dec << ddata << endl;

           //ixbin=khist( (double) x, alow, ahgh, 16);
	   // histogram X distribution for all ntuples
	} // end of 'if ( lineID >= 0 )'
   } // end of forever loop

	// output histograms.
	cout << "Found " << lineTotalLineCount << " lines."  << endl;
	cout << "bufferCount     = " <<  bufferCount << endl;
	cout << "eventCount      = " << eventCount << endl;
	cout << "errorType1count = " << errorType1count << endl;
	cout << "ntupleCount     = " << ntupleCount << endl;

	nout.close();

	return 0;

}
