// reads raw data from run_xxx.txt assigned to link indata
// writes ascii tuple file: ntuple.ascii using nout
#include <iostream>
#include <iomanip>
#include <fstream>              // needed by  ofstream to open output file
#include "histogram.h"
#include <string>
 
#define TAB '\t'
 
/*
RunNumber       RunType EventNumber     BufferSequenceNumber    BufferStartTimeLR       BufferStartTimeHR
      BufferEndTimeLR BufferEndTimeHR
RampTimeAdc     Mcp1TimeAdc     Mcp1PhAdc       Mcp2TimeAdc      Mcp2PhAdc       Mcp3TimeAdc     Mcp3PhAdc       Mcp4TimeAdc     Mcp4PhAdc
Trig1TimeTdc    Trig2TimeTdc    CherenkovTimeTdc
 */
 
using namespace std;
  
unsigned  getbits( unsigned x, unsigned p, unsigned n)
{
    /* get n bits from position p */
        return  (( x >> (p+1-n)) & ~(~0 << n));
}
 

int main(int argc, char** argv)	{

	int i,j,k;
	int lineID, hdata1, ddata1, hdata2, ddata2;
	string titleString;
	int ptrToNext=1;

#define N21
#define EVSIZE 21
#define ADCLOC 0
#define AD1LOC  17
#define AD2LOC  19

/*	definintions for N21
	0	ladc header		8005 8006
	1	adc channel 0
	...
	15	adc channel 14
	16	adc channel 15
	17	AD114 header		8801	8803
	18	AD114 data
	19	AD114 header		8802	8804
	20	AD114 data		
 */
/*
 -5     0       0       0       0       ScalerData Pointer
 -4     11      17      11      17      RealTime LR - Offset
 -3     79ff1   499697  86be7   551911  RealTime HR
 -2     4       4       5       5       CrateID
 -1     3e04    15876   3e04    15876   mPointers
 0      8005    32773   8006    32774   header
 1      c9      201     2e      46
 2      81a     2074    828     2088
 3      101a    4122    102b    4139
 4      1813    6163    181e    6174
 5      201a    8218    202d    8237
 6      281c    10268   2825    10277
 7      301c    12316   3030    12336
 8      3819    14361   382b    14379
 9      4020    16416   402c    16428
 10     481d    18461   4826    18470
 11     501b    20507   502a    20522
 12     5820    22560   5824    22564
 13     601e    24606   6029    24617
 14     681b    26651   682d    26669
 15     701e    28702   7027    28711
 16     7869    30825   79c8    31176
*/

	int inHexData, inDecData;	/* for reading negative index headers */
	int scalerPointer;
	int nScalerValue[4];
	int nRunNumber, nRealTimeLR1, nRealTimeHR1;
	int nout;
	int crateIDc1, crateIDc2;
	int realTimeLRc1, realTimeHRc1;
	int realTimeLRc2, realTimeHRc2;
	

	int memoryPointer1, memoryPointer2, memoryPointer;
	int lineCount;
	int memoryIndex1, memoryIndex2;
	//int data1H[16100], data1D[16100], data2H[16100], data2D[16100];
	int data1H[32200], data1D[32200], data2H[32200], data2D[32200];

	int inIndex[EVSIZE], inHdata1[EVSIZE], inDdata1[EVSIZE], inHdata2[EVSIZE], inDdata2[EVSIZE];
	int inIndexM=0;
	int indexEv=0;
	int offSet1, offSet2;

	int LadcHeader1counter=0, LadcHeader2counter=0;
	int OadcHeader1counter=0, OadcHeader2counter=0, OadcHeader3counter=0, OadcHeader4counter=0;

	int evCountHdr1=0, evCountHdr2=0;


	ifstream in("indata");

	int mBlockCounter=0;

	bool flag1, flag2, flag3, flag4;

	int correctedOadcHeader1, correctedOadcData1;  
	int correctedOadcHeader2, correctedOadcData2;  
	int correctedOadcHeader3, correctedOadcData3;  
	int correctedOadcHeader4, correctedOadcData4;  


   for (;;) {
	mBlockCounter++;
	for ( j=0; j<6; j++) {
		in >> inIndexM >> hex >> inHexData >> dec >> inDecData;
		if ( inIndexM == -6 )  
		{ scalerPointer = inDecData;
		  if ( scalerPointer != 0) {
		    for ( k=0; k< scalerPointer; k++) {
		      in >> inIndexM >> hex >> inHexData >> dec >> inDecData;
		      nScalerValue[k] = inDecData;
		    }
		  }
		}
		if ( inIndexM == -5 )  nRunNumber  = inDecData;
		if ( inIndexM == -4 ) realTimeLRc1 = inDecData;
		if ( inIndexM == -3 ) realTimeHRc1 = inDecData;
		if ( inIndexM == -2 ) crateIDc1 = inDecData;
		if ( inIndexM == -1 ) memoryPointer1 = inDecData;
	}
	if ( mBlockCounter < 3 ) {
		cout << "RunNumber = " << nRunNumber << " realTimeLRc1 = " << realTimeLRc1 << " realTimeHRc1=" << realTimeHRc1 << endl;
		cout << "crateIDc1 = " << crateIDc1 << " memoryPointer1=" << memoryPointer1 << endl;
	}
	// detect an EOF
	if ( in.eof() != 0 ) {
		cout << "Found end of file..." << endl;
		break;
	}
	nRealTimeLR1 = realTimeLRc1;
	nRealTimeHR1 = realTimeHRc1;
	
	if ( crateIDc1 != 4 ) { cerr << "Found wrong camac crate id " << crateIDc1 << " Exit" << endl; return 1;}
	if ( memoryPointer1 != 15876 ) cerr << "Found abnormal memoryPointer1 = " << memoryPointer1 << endl;
	if ( memoryPointer1 >  16100 ) { cerr << "Found bad memoryPointer1 ... Exit" << endl; return 1; }
	for ( j=0; j<memoryPointer1; j++) in >> inIndexM >> hex >> data1H[j] >> dec >> data1D[j];


	for ( j=0; j<6; j++) {
		in >> inIndexM >> hex >> inHexData >> dec >> inDecData;
		if ( inIndexM == -6 )  
		{ scalerPointer = inDecData;
		  if ( scalerPointer != 0) {
		    for ( k=0; k< scalerPointer; k++) {
		      in >> inIndexM >> hex >> inHexData >> dec >> inDecData;
		      nScalerValue[k] = inDecData;
		    }
		  }
		}
		if ( inIndexM == -5 ) nRunNumber = inDecData;
		if ( inIndexM == -4 ) realTimeLRc2 = inDecData;
		if ( inIndexM == -3 ) realTimeHRc2 = inDecData;
		if ( inIndexM == -2 ) crateIDc2 = inDecData;
		if ( inIndexM == -1 ) memoryPointer2 = inDecData;
	}
	if ( mBlockCounter < 3 ) {
		cout << "RunNumber = " << nRunNumber << " realTimeLRc1 = " << realTimeLRc1 << " realTimeHRc1=" << realTimeHRc1 << endl;
		cout << "crateIDc1 = " << crateIDc1 << " memoryPointer1=" << memoryPointer1 << endl;
	}
	if ( crateIDc2 != 5 ) cerr << "Found wrong camac crate id " << crateIDc2 << endl;
	if ( memoryPointer2 != 15876 ) cerr << "Found abnormal memoryPointer2 = " << memoryPointer2 << endl;

	for ( j=0; j<memoryPointer2; j++) in >> inIndexM >> hex >> data2H[j] >> dec >> data2D[j];

	memoryPointer = memoryPointer1;		/* assume pulser run */
	if ( memoryPointer2 > memoryPointer1 ) memoryPointer = memoryPointer2;

	//cout << "memoryPointer1 = " << memoryPointer1 << " memoryPointer2 = " << memoryPointer1 << endl;
	//cout << "crateIDc1 = " << crateIDc1 << " crapteIDc2 = " <<  crateIDc2 << endl;

	for ( j=0; j<63; j++ ) cout << j << TAB << hex << data1H[j] << TAB << dec << data1D[j] 
					<< TAB << hex << data2H[j] << TAB << dec << data2D[j] << endl;

	lineCount = 0;
	int ptr1LadcHdr, ptr1OadcHdr1, ptr1OadcHdr2;
	int ptr2LadcHdr, ptr2OadcHdr1, ptr2OadcHdr2;
	ptr1LadcHdr = 0;
	ptr1OadcHdr1 = 17;
	ptr1OadcHdr2 = 19;
	ptr2LadcHdr = 0;
	ptr2OadcHdr1 = 17;
	ptr2OadcHdr2 = 19;
	memoryIndex1 = 0;	memoryIndex2 = 0;
	
	if ( memoryIndex1 > 16100 ) memoryIndex1 = 15876;
	if ( memoryIndex2 > 16100 ) memoryIndex2 = 15876;

      while  (  (memoryIndex1 < memoryPointer1 ) && (memoryIndex2 < memoryPointer2) )  {

	cout << endl;
	cout << endl;
	cout << "At top of while memoryIndex1=" << memoryIndex1 << " memoryPointer1=" << memoryPointer1
                << " memoryIndex2=" << memoryIndex2 << " memoryPointer2=" << memoryPointer2 << endl;
	cout << "At top of while continued ptr1LadcHdr=" << ptr1LadcHdr << endl;

	
	offSet1 = 21;
	offSet2 = 21;
	for ( i=0; i<21; i++ ) {
                inHdata1[i] = 0;
                inDdata1[i] = 0;
        }

	for ( i=0; i<21; i++ ) {
		inHdata1[i] = data1H[ptr1LadcHdr + i ];
		inDdata1[i] = data1D[ptr1LadcHdr + i ];
		cout << "i="<<i<< TAB << hex << "inHdata1[i]=" << inHdata1[i] << dec << endl;
	}

	flag1=true;
	flag2=true;

/*---------------------------------------------------------------*/
       if ( inHdata1[0] == 0x8005 ) {
                offSet1 = 17;
                LadcHeader1counter++;
        } else {
                cout << "Failed to find 0x8005 header in first memory..so exit" << endl;
                cout << " inHdata1[0] = " << hex << inHdata1[0] << dec <<endl;
                return 1;
        }
        evCountHdr1 = 0;
        if ( inHdata1[17] == 0x8801 && inHdata1[19] == 0x8802 ){
                cout << "Found case 1 " << endl;
                correctedOadcHeader1 = 0x8801;
                correctedOadcData1   = inHdata1[18];
                correctedOadcHeader2 = 0x8802;
                correctedOadcData2   = inHdata1[19];
                offSet1 = 21;
                evCountHdr1=2;
        }
        if ( inHdata1[17] == 0x8801 && inHdata1[19] != 0x8802 ){
                cout << "Found case 2 " << endl;
                correctedOadcHeader1 = 0x8801;
                correctedOadcData1   = inHdata1[18];
                correctedOadcHeader2 = 0x8802;
                correctedOadcData2   = 0;
                offSet1 = 19;
                evCountHdr1=1;
        }
        if ( inHdata1[17] == 0x8802 && inHdata1[19] != 0x8802 ){
                cout << "Found case 3 " << endl;
                correctedOadcHeader1 = 0x8801;
                correctedOadcData1   = 0;
                correctedOadcHeader2 = 0x8802;
                correctedOadcData2   = inHdata1[18];
                offSet1 = 19;
                evCountHdr1=1;
        }
        if ( evCountHdr1 == 0 ) {
                cout << "Found case 4 " << endl;
                correctedOadcHeader1 = 0x8801;
                correctedOadcData1   = 0;
                correctedOadcHeader2 = 0x8802;
                correctedOadcData2   = 0;
                offSet1 = 17;
        }

	inHdata1[17]=correctedOadcHeader1;
	inHdata1[18]=correctedOadcData1;
	inHdata1[19]=correctedOadcHeader2;
	inHdata1[20]=correctedOadcData2;
	
	cout << "At end of tests offSet1=" << offSet1 << endl;

	ptr1LadcHdr = ptr1LadcHdr + offSet1;
	
	//cout << "After Corrections..." << endl;
	//for ( i=0; i<21; i++ ) {
	//	cout << "i="<<i<< TAB << hex << "inHdata1[i]=" << inHdata1[i] << dec << endl;
	//}

// now 2nd memory...

	offSet2=21;

	flag3=true;
	flag4=true;

        for ( i=0; i<21; i++ ) {
                inHdata2[i] = 0;
                inDdata2[i] = 0;
        }
                                                                                                                      
        for ( i=0; i<21; i++ ) {
                inHdata2[i] = data2H[ptr2LadcHdr + i ];
                inDdata2[i] = data2D[ptr2LadcHdr + i ];
		cout << "i="<<i<< TAB << hex << "inHdata2[i]=" << inHdata2[i] << dec << endl;
        }

//--------------------------new
        if ( inHdata2[0] == 0x8006 ) {
                offSet2 = 17;
                LadcHeader2counter++;
        } else {
                cout << "Failed to find 0x8006 header so exit" << endl;
		cout << " inHdata2[0] = " << hex << inHdata2[0] << dec <<endl;
                return 1;
        }
	evCountHdr2 = 0;
	if ( inHdata2[17] == 0x8803 && inHdata2[19] == 0x8804 ){
		cout << "Found case 1 " << endl;
		correctedOadcHeader3 = 0x8803;
		correctedOadcData3   = inHdata2[18];
		correctedOadcHeader4 = 0x8804;
		correctedOadcData4   = inHdata2[20];
		offSet2 = 21;
		evCountHdr2=2;
	}
	if ( inHdata2[17] == 0x8803 && inHdata2[19] != 0x8804 ){
		cout << "Found case 2 " << endl;
		correctedOadcHeader3 = 0x8803;
		correctedOadcData3   = inHdata2[18];
		correctedOadcHeader4 = 0x8804;
		correctedOadcData4   = 0;
		offSet2 = 19;
		evCountHdr2=1;
	}
	if ( inHdata2[17] == 0x8804 && inHdata2[19] != 0x8804 ){
		cout << "Found case 3 " << endl;
		correctedOadcHeader3 = 0x8803;
		correctedOadcData3   = 0;
		correctedOadcHeader4 = 0x8804;
		correctedOadcData4   = inHdata2[18];
		offSet2 = 19;
		evCountHdr2=1;
	}
	if ( evCountHdr2 == 0 )	{
		cout << "Found case 4 " << endl;
		correctedOadcHeader3 = 0x8803;
		correctedOadcData3   = 0;
		correctedOadcHeader4 = 0x8804;
		correctedOadcData4   = 0;
		offSet2 = 17;
	}
	

        inHdata2[17]=correctedOadcHeader3;
        inHdata2[18]=correctedOadcData3;
        inHdata2[19]=correctedOadcHeader4;
        inHdata2[20]=correctedOadcData4;
                                                                                                                                                                   
	cout << "After All Corrections..." << endl;
	for ( i=0; i<21; i++ ) {
		cout << "i="<<i<< TAB << hex << "inHdata1[i]=" << inHdata1[i] 
			<<  TAB << hex << "inHdata2[i]=" << inHdata2[i] << dec << endl;
	}

//--------------------------end new
                                                                                                                      
        ptr2LadcHdr = ptr2LadcHdr + offSet2;

	
	memoryIndex1 = memoryIndex1 + offSet1;
	//memoryIndex2 = memoryIndex2 + 21;
	memoryIndex2 = memoryIndex2 + offSet2;

     } // end of event build loop
   } // end of forever loop

	//int OadcHeader1counter=0, OadcHeader2counter=0, OadcHeader3counter=0, OadcHeader4counter=0;
	cout << "Lecroy Headers: "<< LadcHeader1counter << TAB << LadcHeader2counter << endl;


	return 0;

}
