// reads raw data from run_xxx.txt assigned to link indata
// writes ascii tuple file: ntuple.ascii using nout
#include <iostream>
#include <iomanip>
#include <fstream>		// needed by  ofstream to open output file
#include "histogram.h"
#include <string>

#define TAB '\t'

/*
RunNumber       RunType EventNumber     BufferSequenceNumber    BufferStartTimeLR       BufferStartTimeHR       BufferEndTimeLR BufferEndTimeHR 
RampTimeAdc     Mcp1TimeAdc     Mcp1PhAdc       Mcp2TimeAdc      Mcp2PhAdc       Mcp3TimeAdc     Mcp3PhAdc       Mcp4TimeAdc     Mcp4PhAdc       
Trig1TimeTdc    Trig2TimeTdc    CherenkovTimeTdc
 */

using namespace std;
 
unsigned  getbits( unsigned x, unsigned p, unsigned n)
{
    /* get n bits from position p */
        return  (( x >> (p+1-n)) & ~(~0 << n));
}


int main(int argc, char** argv)	{

	int i;
	int lineID, hdata1, ddata1, hdata2, ddata2;
	int feraID;
	int lineTotalLineCount = 0;
	int fileLineCount = 0;
	int linesInChunk=0;
	int bufferNumber=0, bufferCount=0;
	string titleString;
	int ptrToNext=1;
	bool notTheFirst=false, ptrData=false, firstTime = true ;
	bool ad114Header1 = false;
	bool ad114Header2 = false;
	bool ad114Header3 = false;
	bool ad114Header4 = false;
	int ad114Header1Count=0;
	int ad114Header2Count=0;
	int ad114Header3Count=0;
	int ad114Header4Count=0;
	int ntupleCount=0;
	//unsigned int event[18];
	unsigned int event[28];
	int errorType1count = 0;
	int nRunNumber=0, nEventCount=0;
	int runType=0;
	int bufferSequenceNumber=0;
	int bufferStartTimeLR=0, bufferStartTimeHR=0, bufferEndTimeLR=0, bufferEndTimeHR=0;
	int rampTimeAdc=0; 
	int trig1TimeTdc=0, trig2TimeTdc=0, cherenkovTimeTdc=0;
	int nADC1time = 0, nADC2time = 0,  nADC1ph = 0, nADC2ph = 0;
	int nADC3time = 0, nADC4time = 0,  nADC3ph = 0, nADC4ph = 0;

#define DSIZE 15876
	int index0[DSIZE], inHdata1s[DSIZE], inDdata1s[DSIZE], inHdata1s[DSIZE], inDdata1s[DSIZE];

#define N21
#define EVSIZE 21
#define ADCLOC 0
#define AD1LOC  17
#define AD2LOC  19

/*
#define N19
#define EVSIZE 19
#define ADCLOC 0
#define AD1LOC  17
#define AD2LOC  -1
 */

/*	definintions for N21
	0	ladc header		8005 8006
	1	adc channel 0
	...
	15	adc channel 14
	16	adc channel 15
	17	AD114 header		8801	8803
	18	AD114 data
	19	AD114 header		8802	8804
	20	AD114 data		
 */
	int inIndex[EVSIZE], inHdata1[EVSIZE], inDdata1[EVSIZE], inHdata2[EVSIZE], inDdata2[EVSIZE];
	int indexEv=0;
	int countADCheader1=0, countADCheader2=0;
	int countAD114header1=0, countAD114header2=0;
	int countAD114header3=0, countAD114header4=0;
	int lineCount=0, eventCount=0;
	int lAdc1ChAddr[16], lAdc2ChAddr[16];
	int lAdc1ChData[16], lAdc2ChData[16];

	float mADCtime1 = 3050.0;
	float lADCtime1 = 500.0;
	//histogram *hADCtime1 = new histogram( mADCtime1-lADCtime1, mADCtime1+lADCtime1, "h time ADC 1 data ");
	histogram *hADCtime1 = new histogram( 0.0, 5000.0, "h time ADC 1 data ");
	float mADCtime2 = 1850.0;
	float lADCtime2 = 500.0;
	//histogram *hADCtime2 = new histogram( mADCtime2-lADCtime2, mADCtime2+lADCtime2, "h time ADC 2 data ");
	histogram *hADCtime2 = new histogram( 0.0, 5000.0, "h time ADC 2 data ");

	histogram *hADCtime3 = new histogram( 0.0, 5000.0, "h time ADC 3 data ");
	histogram *hADCtime4 = new histogram( 0.0, 5000.0, "h time ADC 4 data ");

	 histogram *hADCpm1 = new histogram(0.0, 400.0, "h ph ADC 1 data ");
	 histogram *hADCpm2 = new histogram(0.0, 400.0, "h ph ADC 2 data ");
	 histogram *hADCpm3 = new histogram(0.0, 400.0, "h ph ADC 3 data ");
	 histogram *hADCpm4 = new histogram(0.0, 400.0, "h ph ADC 4 data ");

	//cout << "Enter Run Description parameters: " << endl;
	//cout <<"runNumber  pulserDelay  pulserPeriod " << endl;
	//cin >> runNumber >> pulserDelay >> pulserPeriod;


	ifstream in("indata");


	// 1st line is text not data
	//getline(in,titleString);
	//fileLineCount++;
	
	cout << "Title: " << titleString << endl;
	cout << "DEBUG:\tLine\tlineID\thdata\tddata" << endl;

   // open an output stream to make file of ascii nutples for ROOT
   ofstream nout("ntuple.ascii");
   if(!nout)
   {
     cout << "Cannot open output file.\n";
     return 1;
   }
	// output column names
	nout << "RunNumber"<<TAB<<"RunType"<< TAB<< "EventNumber" <<TAB<< "BufferSequenceNumber" 
			<<TAB<<"BufferStartTimeLR"<<TAB<< "BufferStartTimeHR"<<TAB<<"BufferEndTimeLR"<<TAB<<"BufferEndTimeHR"
			<<TAB<<"RampTimeAdc"
			<<TAB<<"Mcp1TimeAdc"<<TAB<< "Mcp1PhAdc" <<TAB<<"Mcp2TimeAdc"<<TAB<<"Mcp2PhAdc"
			<<TAB<<"Mcp3TimeAdc"<<TAB<< "Mcp3PhAdc" <<TAB<<"Mcp4TimeAdc"<<TAB<<"Mcp4PhAdc"
	                <<TAB<<"Trig1TimeTdc"<<TAB<<"Trig2TimeTdc"<< TAB << "CherenkovTimeTdc" <<endl;

   for ( i=0; i<DSIZE; i++)
	in >> index0[i]  >> inHdata1s[i] >> inDdata1s[i] >> inHdata2s[i] >> inDdata2s[i]


   for (;;) {
	
	lineCount++;

	if ( in.eof() != 0 ) {
		cout << "Found end of file..." << endl;
		break;
	}

           //ixbin=khist( (double) x, alow, ahgh, 16);
	   // histogram X distribution for all ntuples
   } // end of forever loop

	// output histograms.
	hADCtime1->printLong();
	hADCtime2->printLong();
	hADCtime3->printLong();
	hADCtime4->printLong();
	hADCpm1->printLong();
	hADCpm2->printLong();
	hADCpm3->printLong();
	hADCpm4->printLong();

	cout << endl;
	cout << "Found " << lineCount << " lines."  << endl;
	cout << "eventCount      = " << eventCount << endl;
	cout << "errorType1count = " << errorType1count << endl;
	cout << "countADCheader1 = " << countADCheader1 << endl;
	cout << "countADCheader2 = " << countADCheader2 << endl;
	cout << "countAD114header1 = " << countAD114header1 << endl;
	cout << "countAD114header2 = " << countAD114header2 << endl;
	cout << "ntupleCount     = " << ntupleCount << endl;

	nout.close();

	return 0;

}
