// program used to read raw data files and produce hardware ntuple files
// July 14, 2008 Ed May
// retrieved from atlas2.hep.anl.gov... used to process all t979 run 1 data

// read t979 rawdata file run_0NN.txt from inRaw
// write to outfile  file run_0NN.tup via outTup
// inRaw and outTup assigned a logical file name/ptr via processing script(s)

#include <stdio.h>
#include <stdlib.h>
#define DLINES 1000000
#define DEBUG1 1
#define DEBUG2 0


unsigned  getbits( unsigned x, unsigned p, unsigned n)
{
    /* get n bits from position p */
        return  (( x >> (p+1-n)) & ~(~0 << n));
}


int main()
{
    FILE *data_file,*output_file, *output_file2;
    char infile[20]  = ("inRaw");
    char outfile[20] = ("outTup");
    char outfile2[20] = ("outCsv");
    int data1,data2,data3;
    //unsigned int adcdata[16];
    int adcdata[16];
    int i,j;
    int TAC1data,TAC2data,TAC3data;
    int foundTAC1,foundTAC2,foundTAC3;
    int ioret;
    int lineCount=0;
    int countTAC1=0, countTAC2=0, countTAC3=0;
    int countMultiTAC1=0, countMultiTAC2=0, countMultiTAC3=0;
    int count4300Header=0;
    int blockCount=0;
    int iMax=0, iscaler=0, scalerTotal[4]={0,0,0,0}, scalerValue[4];
    int runType=0, runNumb=0, lrClock=0, hrClock=0, crateID=0, blockSize=0;
    unsigned int usInt=0; 
    unsigned int chanCount=0;
    unsigned int chanID=0;
    unsigned int adcdataIn[16];
    int firstTime=1; // is true
    int tupleNum=0;
    int iblk=0;	// index for reading raw data blocks
    int channelPop[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    int missingAdcChFlag = 0; // counts number of missing adc channels in each event
    int multiTACflag = 0;


    data_file = fopen(infile,"r");	// infile is char array  containing inRaw 
    if ( data_file == NULL ) {
      printf("Failed to open an input file!\n");
      return 1;}
    output_file = fopen(outfile,"w");	// outfile is char array containing outTup
    if ( output_file == NULL ) {
      printf("Failed to open an output file!\n");
      return 1;}
    output_file2 = fopen(outfile2,"a");	// outfile is char array containing outCsv
    if ( output_file2 == NULL ) {
      printf("Failed to open an output2 file!\n");
      return 1;}

	/* reset event */
    adcdata[0]=0; adcdata[1]=0; adcdata[2]=0; adcdata[3]=0; 
    adcdata[4]=0; adcdata[5]=0; adcdata[6]=0;
    TAC1data=0; TAC2data=0; TAC3data=0;
    foundTAC1=0; foundTAC2=0; foundTAC3=0;
    runNumb=0; runType=0; lrClock=0; hrClock=0; blockSize=0;
   
    //fprintf(output_file, "adc1 adc2 tac1 tac2 tac3\n");
    //fprintf(output_file, "ramp1\tramp2\tadc1\tadc2\ttac1\ttac2\ttac3\n");
    fprintf(output_file,"runType\trunNum\tblkCnt\ttuplNum\tscaler1\tscaler2\tscaler3\tramp1\tramp2\tadc1\tadc2\ttac1\ttac2\ttac3\tadc3\tadc4\tadc5\thrClock\tlrClock\n");

// Read raw data blocks and do integrety checks... 

  for (;;) {
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( ioret == EOF ) {printf("EOF found for run %d, break from for loop.\n",runNumb); break;}
	if ( data1 != -7 ) {
		printf("Failed to find Scaler header!\n");
		printf("lineCount=%d Data= %d %x %d\n", lineCount, data1, data2, data3);
		printf("Exit!\n");
		return 2;
	} else {
		blockCount++;
		iMax=data3;	// how many scaler elements
		for ( iscaler=0; iscaler<iMax; iscaler++ ) {
		      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      			lineCount++;
			//printf("READ %d %x %d\n",data1,data2,data3);
		      scalerValue[iscaler] = data3;
		      scalerTotal[iscaler] = scalerTotal[iscaler] + data3;
		}
		printf("Found block header and %d Scalers: %d %d %d %d\n", 
			iMax, scalerValue[0],scalerValue[1],scalerValue[2],scalerValue[3]);
			
	}
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -6 ) {
		printf("Failed to find runType header!\n");
                return 2;
        } else {
		runType = data3;
      }
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -5 ) {
		printf("Failed to find runNumb header!\n");
                return 2;
        } else {
		runNumb = data3;
      }
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -4 ) {
		printf("Failed to find Low Resolution clock header!\n");
                return 2;
        } else {
		lrClock = data3;
      }
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -3 ) {
		printf("Failed to find High Resolution clock header!\n");
                return 2;
        } else {
		hrClock = data3;
      }
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -2 ) {
		printf("Failed to find Camac Crate ID header!\n");
                return 2;
        } else {
		crateID = data3;
      }
      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      if ( data1 != -1 ) {
		printf("Failed to find FERA Memory header!\n");
                return 2;
        } else {
		blockSize = data3;
      }
      printf("Block Header reads complete: runType/runNumb/lrClock/hrClock/crateID/blockSize = %d %d %d %d %d %d\n", runType, runNumb, lrClock, hrClock, crateID, blockSize);
		
		
	

    /*  Begin reading a FERA memory block and building events   */

      int memPtr=0;
      while ( memPtr<blockSize-1 ) {

      ioret = fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
      lineCount++;
      memPtr++;
      usInt = data3 & 0x8005;	// allow header with missing channels
      if (usInt == 0x8005 ) {
	/* Found new generalized 4300 ADC data - print out last event */
        count4300Header++;
	if ( firstTime ) { firstTime=0; }
	else { 
	  //fprintf(output_file,"%d %d %d %d %d\n",
          //       adcdata[2],adcdata[3],TAC1data,TAC2data,TAC3data);
	  if ( missingAdcChFlag <= 8 && multiTACflag == 0 ) {
	  tupleNum++;
	  //                    1   2   3   4   5   6   7   8   9   0   1   2   3   4   5   6
	   //printf("DEBUG LADC: %d\t%d\t%d\t%d\t%d\t%d\t%d \n", 
	   // adcdata[0],adcdata[1],adcdata[2],adcdata[3],adcdata[4],adcdata[5],adcdata[6]);
	  fprintf(output_file,"%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
			runType,runNumb,blockCount,tupleNum,
			scalerValue[1],scalerValue[2],scalerValue[3],
                        adcdata[0],adcdata[1],adcdata[2],adcdata[3],
			TAC1data,TAC2data,TAC3data,
			adcdata[4],adcdata[5],adcdata[6],
			hrClock,lrClock);
	  }
	} // end of firstTime check

	// extract the channel count
	chanCount=getbits( (unsigned int) data3, 14, 4);
	if ( chanCount == 0 ) chanCount=16;
	missingAdcChFlag = 16-chanCount;
	multiTACflag = 0;


	/* reset event */
        TAC1data=0; TAC2data=0; TAC3data=0;
	foundTAC1=0; foundTAC2=0; foundTAC3=0;

	/*  Read in new ADC data */
	for(j=0;j<16;j++){ adcdata[j]=-1;}
	for(j=0;j<chanCount;j++){
	  fscanf(data_file,"%d %x %d",&data1,&data2,&adcdataIn[j]);
           memPtr++;
	  lineCount++;

	  chanID = getbits( (unsigned int) adcdataIn[j], 14, 4);
	  channelPop[chanID]++;
	  adcdata[chanID] = adcdataIn[chanID] & 0x7FF;

	  //printf("DEBUG j=%d adcdataIn[j]=%X chanID=%d adcdata[chanID]=%X (hex) adcdata[chanID]=%d (dec)\n", 
	//	j,adcdataIn[j], chanID, adcdata[chanID], adcdata[chanID]);

	  if((data3==34817 || data3==34818 || data3==34819)) {
	    printf(" Error!! At lineCount=%d Found header in middle of ADC = %x",
		lineCount,data3);
	    return 2; }
	}
      } // end of if(data3==32773)

      /* Look for TAC headers.  If found already, and no new 4300 data,
	 then something wrong with event and set to 0 */

      if(data3==34817){
	fscanf(data_file,"%d %x %d",&data1,&data2,&data3); 
        memPtr++;
	  lineCount++;
        if(foundTAC1) {
	  TAC1data=0; 
	  if (DEBUG2) printf(" At lineCount = %d index=%d Multiple TAC1 \n", lineCount,data1); 
	  countMultiTAC1++;
          multiTACflag++;
        }
        else TAC1data=data3; 
        foundTAC1 = 1; countTAC1++; 
      } // end of if(data3==34817) ie TAC1

      if(data3==34818){
	fscanf(data_file,"%d %x %d",&data1,&data2,&data3);
        memPtr++;
	  lineCount++;
        if(foundTAC2) {
	  TAC2data=0;
	  if (DEBUG2) printf(" At lineCount = %d index=%d Multiple TAC2 \n", lineCount,data1); 
	  countMultiTAC2++;
          multiTACflag++;
        }
        else TAC2data=data3; 
        foundTAC2 = 1; countTAC2++; 
      } // end of if(data3==34818) ie TAC2

      if(data3==34819){
	fscanf(data_file,"%d %x %d",&data1,&data2,&data3); 
        memPtr++;
	  lineCount++;
        if(foundTAC3) {
	  TAC3data=0; 
	  if (DEBUG2) printf(" At lineCount = %d index=%d Multiple TAC3 \n", lineCount,data1); 
	  countMultiTAC3++;
          multiTACflag++;
        }
        else TAC3data=data3; 
        foundTAC3 = 1; countTAC3++; 
      } // end of if(data3==34819) ie TAC3

    } // end of while loop on blockSize
	// remember to write out the last event...
	if ( missingAdcChFlag <= 8 && multiTACflag == 0) {
	    tupleNum++;
	    //                    1   2   3   4   5   6   7   8   9   0   1   2   3   4   5   6
	    fprintf(output_file,"%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
			runType,runNumb,blockCount,tupleNum,
			scalerValue[1],scalerValue[2],scalerValue[3],
                        adcdata[0],adcdata[1],adcdata[2],adcdata[3],
			TAC1data,TAC2data,TAC3data,
			adcdata[4],adcdata[5],adcdata[6],
			hrClock,lrClock);
	}
  } // end of forever

    printf("Number of tuples written out: %d \n", tupleNum);
    printf("Found %d Memory Blocks\n", blockCount);
    printf("Found %d 4300 headers\n", count4300Header);
    printf("Found countTAC1 = %d \n", countTAC1);
    printf("Found countTAC2 = %d \n", countTAC2);
    printf("Found countTAC3 = %d \n", countTAC3);
    printf("Found %d MultiTAC1headers or %f (\%)\n", countMultiTAC1,(countMultiTAC1*100.0/count4300Header));
    printf("Found %d MultiTAC1headers or %f (\%)\n", countMultiTAC2,(countMultiTAC2*100.0/count4300Header));
    printf("Found %d MultiTAC1headers or %f (\%)\n", countMultiTAC3,(countMultiTAC3*100.0/count4300Header));
    printf("Found scaler[0] sum = %d\n", scalerTotal[0]);
    printf("Found scaler[1] sum = %d\n", scalerTotal[1]);
    printf("Found scaler[2] sum = %d\n", scalerTotal[2]);
    printf("Found scaler[3] sum = %d\n", scalerTotal[3]);
    printf("Livetime: %f \n", 100.0*scalerTotal[2]/scalerTotal[1]);
    printf("LeCroy ADC channel population: ");
    for (i=0;i<16;i++) printf("%d ",channelPop[i]);
    printf("\n");
    printf("LeCroy ADC channel inefficencies (\%): ");
    for (i=0;i<16;i++) printf("%6.3f ", ((count4300Header-channelPop[i])*100.0/count4300Header) );
    printf("\n");

//    fprintf(output_file2,"%s%s%s%s\n", "runNumb,blockCount,count4300Header,countTAC1,countTAC2,countTAC3,",
//    "countMultiTAC1,countMultiTAC2,countMultiTAC3,scalerTotal1,scalerTotal2,scalerTotal3,",
//    "channelPop0,channelPop1,channelPop2,channelPop3,channelPop4,channelPop5,channelPop6,channelPopr72,",
//    "channelPop8,channelPop9,channelPop10,channelPop11,channelPop12,channelPop13,channelPop14,channelPop15");
    fprintf(output_file2,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
	runNumb, blockCount, count4300Header, countTAC1, countTAC2, countTAC3,
	countMultiTAC1, countMultiTAC2, countMultiTAC3, scalerTotal[1], scalerTotal[2], scalerTotal[3]);
    for (i=0;i<16;i++) fprintf(output_file2,",%d",channelPop[i]);
    fprintf(output_file2,"\n");

    printf("Found EOF...exiting.\n");

}
