// reads raw data from run_xxx.txt assigned to link indata
// writes ascii tuple file: ntuple.ascii using nout
#include <iostream>
#include <iomanip>
#include <fstream>		// needed by  ofstream to open output file
#include "histogram.h"
#include <string>

#define TAB '\t'

/*
RunNumber       RunType EventNumber     BufferSequenceNumber    BufferStartTimeLR       BufferStartTimeHR       BufferEndTimeLR BufferEndTimeHR 
RampTimeAdc     Mcp1TimeAdc     Mcp1PhAdc       Mcp2TimeAdc      Mcp2PhAdc       Mcp3TimeAdc     Mcp3PhAdc       Mcp4TimeAdc     Mcp4PhAdc       
Trig1TimeTdc    Trig2TimeTdc    CherenkovTimeTdc
 */

using namespace std;
 
unsigned  getbits( unsigned x, unsigned p, unsigned n)
{
    /* get n bits from position p */
        return  (( x >> (p+1-n)) & ~(~0 << n));
}


int main(int argc, char** argv)	{

	int i;
	int lineID, hdata1, ddata1, hdata2, ddata2;
	int feraID;
	int lineTotalLineCount = 0;
	int fileLineCount = 0;
	int linesInChunk=0;
	int bufferNumber=0, bufferCount=0;
	string titleString;
	int ptrToNext=1;
	bool notTheFirst=false, ptrData=false, firstTime = true ;
	bool ad114Header1 = false;
	bool ad114Header2 = false;
	bool ad114Header3 = false;
	bool ad114Header4 = false;
	int ad114Header1Count=0;
	int ad114Header2Count=0;
	int ad114Header3Count=0;
	int ad114Header4Count=0;
	int ntupleCount=0;
	//unsigned int event[18];
	unsigned int event[28];
	int errorType1count = 0;
	int nRunNumber=0, nEventCount=0;
	int runType=0;
	int bufferSequenceNumber=0;
	int bufferStartTimeLR=0, bufferStartTimeHR=0, bufferEndTimeLR=0, bufferEndTimeHR=0;
	int slowRampTimeAdc=0, fastRampTimeAdc=0, nRampTimeAdc=0; 
	int trig1TimeTdc=0, trig2TimeTdc=0, cherenkovTimeTdc=0;
	int nADC1time = 0, nADC2time = 0,  nADC1ph = 0, nADC2ph = 0;
	int nADC3time = 0, nADC4time = 0,  nADC3ph = 0, nADC4ph = 0;

#define N21
#define EVSIZE 21
#define ADCLOC 0
#define AD1LOC  17
#define AD2LOC  19

/*
#define N19
#define EVSIZE 19
#define ADCLOC 0
#define AD1LOC  17
#define AD2LOC  -1
 */

/*	definintions for N21
	0	ladc header		8005 8006
	1	adc channel 0
	...
	15	adc channel 14
	16	adc channel 15
	17	AD114 header		8801	8803
	18	AD114 data
	19	AD114 header		8802	8804
	20	AD114 data		
 */
	int inIndex[EVSIZE], inHdata1[EVSIZE], inDdata1[EVSIZE], inHdata2[EVSIZE], inDdata2[EVSIZE];
	int indexEv=0;
	int countADCheader1=0, countADCheader2=0;
	int countAD114header1=0, countAD114header2=0;
	int countAD114header3=0, countAD114header4=0;
	int lineCount=0, eventCount=0;
	int lAdc1ChAddr[16], lAdc2ChAddr[16];
	int lAdc1ChData[16], lAdc2ChData[16];

	float mADCtime1 = 3050.0;
	float lADCtime1 = 500.0;
	//histogram *hADCtime1 = new histogram( mADCtime1-lADCtime1, mADCtime1+lADCtime1, "h time ADC 1 data ");
	histogram *hADCtime1 = new histogram( 0.0, 5000.0, "h time ADC 1 data ");
	float mADCtime2 = 1850.0;
	float lADCtime2 = 500.0;
	//histogram *hADCtime2 = new histogram( mADCtime2-lADCtime2, mADCtime2+lADCtime2, "h time ADC 2 data ");
	histogram *hADCtime2 = new histogram( 0.0, 5000.0, "h time ADC 2 data ");

	histogram *hADCtime3 = new histogram( 0.0, 5000.0, "h time ADC 3 data ");
	histogram *hADCtime4 = new histogram( 0.0, 5000.0, "h time ADC 4 data ");

	 histogram *hADCpm1 = new histogram(0.0, 400.0, "h ph ADC 1 data ");
	 histogram *hADCpm2 = new histogram(0.0, 400.0, "h ph ADC 2 data ");
	 histogram *hADCpm3 = new histogram(0.0, 400.0, "h ph ADC 3 data ");
	 histogram *hADCpm4 = new histogram(0.0, 400.0, "h ph ADC 4 data ");
	int slowRampValue, fastRampValue;
//	adc is 10 bits so max is 1024
	 histogram *hSlowRamp = new histogram(0.0, 600.0 , "h Slow Ramp data ");
	 histogram *hFastRamp = new histogram(0.0, 600.0 , "h Fast Ramp data ");

	//cout << "Enter Run Description parameters: " << endl;
	//cout <<"runNumber  pulserDelay  pulserPeriod " << endl;
	//cin >> runNumber >> pulserDelay >> pulserPeriod;


	ifstream in("indata");


	// 1st line is text not data
	//getline(in,titleString);
	//fileLineCount++;
	
	cout << "Title: " << titleString << endl;
	cout << "DEBUG:\tLine\tlineID\thdata\tddata" << endl;

   // open an output stream to make file of ascii nutples for ROOT
   ofstream nout("ntuple.ascii");
   if(!nout)
   {
     cout << "Cannot open output file.\n";
     return 1;
   }
	// output column names
	nout << "RunNumber"<<TAB<<"RunType"<< TAB<< "EventNumber" <<TAB<< "BufferSequenceNumber" 
			<<TAB<<"BufferStartTimeLR"<<TAB<< "BufferStartTimeHR"<<TAB<<"BufferEndTimeLR"
			<<TAB<<"LrRampTimeAdc" <<TAB<<"HrRampTimeAdc"
			<<TAB<<"Mcp1TimeAdc"<<TAB<< "Mcp1PhAdc" <<TAB<<"Mcp2TimeAdc"<<TAB<<"Mcp2PhAdc"
			<<TAB<<"Mcp3TimeAdc"<<TAB<< "Mcp3PhAdc" <<TAB<<"Mcp4TimeAdc"<<TAB<<"Mcp4PhAdc"
	                <<TAB<<"Trig1TimeTdc"<<TAB<<"Trig2TimeTdc"<< TAB << "CherenkovTimeTdc" <<endl;
/*
RunNumber       RunType EventNumber     BufferSequenceNumber    BufferStartTimeLR       BufferStartTimeHR       BufferEndTimeLR BufferEndTimeHR
RampTimeAdc     Mcp1TimeAdc     Mcp1PhAdc       Mcp2TimeAdc      Mcp2PhAdc       Mcp3TimeAdc     Mcp3PhAdc       Mcp4TimeAdc     Mcp4PhAdc Trig1TimeTdc    Trig2TimeTdc    CherenkovTimeTdc
 */



   for (;;) {
	
	lineCount++;
	indexEv = (lineCount - 1)%EVSIZE;

	in >> inIndex[indexEv] >> hex >> inHdata1[indexEv] >> dec >> inDdata2[indexEv] 
		>> hex >> inHdata2[indexEv]>>dec>>inDdata2[indexEv];

	if ( lineCount <= 32 ) cout<<"Line: "<< lineCount << TAB << indexEv << TAB 
			<< inIndex[indexEv] << TAB << hex << inHdata1[indexEv] << TAB<<dec << inDdata1[indexEv]
			                    << TAB << hex << inHdata2[indexEv] << TAB<<dec << inDdata2[indexEv]
			<< endl;
	if ( inHdata1[0] == 0 && lineCount == 1) countADCheader1++;	/* kludge to get sums correct */

	if ( indexEv == 0 && lineCount !=1 ) {

		int LeCroyADCheader1 = 0x8004;
		    LeCroyADCheader1 = 0x8005;	/* did this change */
		int LeCroyADCheader2 = 0x8006;

		if ( (inHdata1[ADCLOC] != LeCroyADCheader1) && (inHdata2[ADCLOC] != LeCroyADCheader2)) { /* lecroy 4300 ADC header */
			cout << "at line = "<<  lineCount <<  " ADC Alignment error..." << endl;
			return 1;
		} else {
			countADCheader1++;
			countADCheader2++;
		}
                if ( (inHdata1[AD1LOC] != 0x8801) && (inHdata2[AD1LOC] != 0x8803) ) {
                        cout << "ad114 Alignment error on 8801..." << endl;
                } else {
                        countAD114header1++;
                        countAD114header3++;
                        nADC1time = inHdata1[AD1LOC+1];
                        nADC3time = inHdata2[AD1LOC+1];
                }
#ifdef N21
                if ( (inHdata1[AD2LOC] != 0x8802) && (inHdata2[AD1LOC] != 0x8804) ) {
                        cout << "ad114 Alignment error on 8802..." << endl;
                } else {
                        countAD114header2++;
                        countAD114header4++;
                        nADC2time = inHdata1[AD2LOC+1];
                        nADC4time = inHdata2[AD2LOC+1];
                }
#endif

		eventCount++;
		for ( i=0; i<16; i++ ) {
			lAdc1ChAddr[i] = getbits(inHdata1[ADCLOC+1+i],15,5);
			lAdc1ChData[i] = getbits(inHdata1[ADCLOC+1+i],10,10);
			lAdc2ChAddr[i] = getbits(inHdata2[ADCLOC+1+i],15,5);
			lAdc2ChData[i] = getbits(inHdata2[ADCLOC+1+i],10,10);
		}
		
			hADCtime1->update( (float) inHdata1[AD1LOC+1] );
			hADCtime2->update( (float) inHdata1[AD2LOC+1] );
			hADCtime3->update( (float) inHdata2[AD2LOC+1] );
			hADCtime4->update( (float) inHdata2[AD2LOC+1] );
		hADCpm1->update( (float) lAdc1ChData[0] );
		hADCpm2->update( (float) lAdc1ChData[15] );
		hADCpm3->update( (float) lAdc2ChData[0] );
		hADCpm4->update( (float) lAdc2ChData[15] );
		slowRampValue = lAdc1ChData[1];
		fastRampValue = lAdc1ChData[2];
		nRampTimeAdc = slowRampValue;			/* choose only one for current ntuple */
		hSlowRamp->update( (float) slowRampValue );
		hFastRamp->update( (float) fastRampValue );
		if ( eventCount < 10 ) {
			for ( i=0; i<16; i++ ) {
				cout << " " << lAdc1ChAddr[i] 
				<< " "      << lAdc1ChData[i]
				<< " "      << lAdc2ChAddr[i]
				<< " "      << lAdc2ChData[i];
			}
			cout << endl;
		}
		// begin building ntuple to outup
		nADC1ph = lAdc1ChData[0];
		nADC2ph = lAdc1ChData[15];
		nADC3ph = lAdc2ChData[0];
		nADC4ph = lAdc2ChData[15];
		nEventCount = eventCount;
		nout << nRunNumber << TAB<< runType<<TAB << nEventCount <<TAB<< bufferSequenceNumber 
		<<TAB<< bufferStartTimeLR <<TAB<< bufferStartTimeHR <<TAB<< bufferEndTimeLR
		<<TAB<< slowRampValue << TAB <<  fastRampValue
		<<TAB<< nADC1time <<TAB<<  nADC1ph <<TAB<< nADC2time <<TAB<< nADC2ph
		<<TAB<< nADC3time <<TAB<<  nADC3ph <<TAB<< nADC4time <<TAB<< nADC4ph
		<<TAB<< trig1TimeTdc <<TAB<< trig2TimeTdc <<TAB<<  cherenkovTimeTdc
		                                        << endl;
	}

/*
RunNumber       RunType EventNumber     BufferSequenceNumber    BufferStartTimeLR       BufferStartTimeHR       BufferEndTimeLR BufferEndTimeHR
RampTimeAdc     Mcp1TimeAdc     Mcp1PhAdc       Mcp2TimeAdc      Mcp2PhAdc       Mcp3TimeAdc     Mcp3PhAdc       Mcp4TimeAdc     Mcp4PhAdc Trig1TimeTdc    Trig2TimeTdc    CherenkovTimeTdc
 */


		

	// detect an EOF
	if ( in.eof() != 0 ) {
		cout << "Found end of file..." << endl;
		break;
	}

           //ixbin=khist( (double) x, alow, ahgh, 16);
	   // histogram X distribution for all ntuples
   } // end of forever loop

	// output histograms.
	hADCtime1->printLong();
	hADCtime2->printLong();
	hADCtime3->printLong();
	hADCtime4->printLong();
	hADCpm1->printLong();
	hADCpm2->printLong();
	hADCpm3->printLong();
	hADCpm4->printLong();
	hSlowRamp->printLong();
	hFastRamp->printLong();

	cout << endl;
	cout << "Found " << lineCount << " lines."  << endl;
	cout << "eventCount      = " << eventCount << endl;
	cout << "errorType1count = " << errorType1count << endl;
	cout << "countADCheader1 = " << countADCheader1 << endl;
	cout << "countADCheader2 = " << countADCheader2 << endl;
	cout << "countAD114header1 = " << countAD114header1 << endl;
	cout << "countAD114header2 = " << countAD114header2 << endl;
	cout << "ntupleCount     = " << ntupleCount << endl;

	nout.close();

	return 0;

}
